@@ -10,21 +10,32 @@ this distribution for specifics.
 
 NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
 Pearu Peterson
+
 """
+from __future__ import division, absolute_import, print_function
 
 __all__ = ['cpu']
 
 import sys, re, types
 import os
-import commands
+
+if sys.version_info[0] >= 3:
+    from subprocess import getstatusoutput
+else:
+    from commands import getstatusoutput
+
 import warnings
+import platform
+
+from numpy.distutils.compat import get_exception
 
 def getoutput(cmd, successful_status=(0,), stacklevel=1):
     try:
-        status, output = commands.getstatusoutput(cmd)
-    except EnvironmentError, e:
+        status, output = getstatusoutput(cmd)
+    except EnvironmentError:
+        e = get_exception()
         warnings.warn(str(e), UserWarning, stacklevel=stacklevel)
-        return False, output
+        return False, ""
     if os.WIFEXITED(status) and os.WEXITSTATUS(status) in successful_status:
         return True, output
     return False, output
@@ -61,27 +72,35 @@ class CPUInfoBase(object):
     the availability of various CPU features.
     """
 
-    def _try_call(self,func):
+    def _try_call(self, func):
         try:
             return func()
-        except:
+        except Exception:
             pass
 
-    def __getattr__(self,name):
+    def __getattr__(self, name):
         if not name.startswith('_'):
-            if hasattr(self,'_'+name):
-                attr = getattr(self,'_'+name)
-                if type(attr) is types.MethodType:
+            if hasattr(self, '_'+name):
+                attr = getattr(self, '_'+name)
+                if isinstance(attr, types.MethodType):
                     return lambda func=self._try_call,attr=attr : func(attr)
             else:
                 return lambda : None
-        raise AttributeError,name
+        raise AttributeError(name)
 
     def _getNCPUs(self):
         return 1
 
+    def __get_nbits(self):
+        abits = platform.architecture()[0]
+        nbits = re.compile(r'(\d+)bit').search(abits).group(1)
+        return nbits
+
     def _is_32bit(self):
-        return not self.is_64bit()
+        return self.__get_nbits() == '32'
+
+    def _is_64bit(self):
+        return self.__get_nbits() == '64'
 
 class LinuxCPUInfo(CPUInfoBase):
 
@@ -96,15 +115,16 @@ class LinuxCPUInfo(CPUInfoBase):
             info[0]['uname_m'] = output.strip()
         try:
             fo = open('/proc/cpuinfo')
-        except EnvironmentError, e:
-            warnings.warn(str(e), UserWarning)
+        except EnvironmentError:
+            e = get_exception()
+            warnings.warn(str(e), UserWarning, stacklevel=2)
         else:
             for line in fo:
                 name_value = [s.strip() for s in line.split(':', 1)]
                 if len(name_value) != 2:
                     continue
                 name, value = name_value
-                if not info or info[-1].has_key(name): # next processor
+                if not info or name in info[-1]: # next processor
                     info.append({})
                 info[-1][name] = value
             fo.close()
@@ -124,10 +144,10 @@ class LinuxCPUInfo(CPUInfoBase):
         return self._is_AMD() and self.info[0]['model'] == '3'
 
     def _is_AthlonK6(self):
-        return re.match(r'.*?AMD-K6',self.info[0]['model name']) is not None
+        return re.match(r'.*?AMD-K6', self.info[0]['model name']) is not None
 
     def _is_AthlonK7(self):
-        return re.match(r'.*?AMD-K7',self.info[0]['model name']) is not None
+        return re.match(r'.*?AMD-K7', self.info[0]['model name']) is not None
 
     def _is_AthlonMP(self):
         return re.match(r'.*?Athlon\(tm\) MP\b',
@@ -222,7 +242,11 @@ class LinuxCPUInfo(CPUInfoBase):
         return self.is_PentiumIV() and self.has_sse3()
 
     def _is_Nocona(self):
-        return self.is_64bit() and self.is_PentiumIV()
+        return self.is_Intel() \
+               and (self.info[0]['cpu family'] == '6' \
+                    or self.info[0]['cpu family'] == '15' ) \
+               and (self.has_sse3() and not self.has_ssse3())\
+               and re.match(r'.*?\blm\b', self.info[0]['flags']) is not None
 
     def _is_Core2(self):
         return self.is_64bit() and self.is_Intel() and \
@@ -235,7 +259,7 @@ class LinuxCPUInfo(CPUInfoBase):
 
     def _is_XEON(self):
         return re.match(r'.*?XEON\b',
-                        self.info[0]['model name'],re.IGNORECASE) is not None
+                        self.info[0]['model name'], re.IGNORECASE) is not None
 
     _is_Xeon = _is_XEON
 
@@ -254,36 +278,25 @@ class LinuxCPUInfo(CPUInfoBase):
         return self.info[0]['f00f_bug']=='yes'
 
     def _has_mmx(self):
-        return re.match(r'.*?\bmmx\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bmmx\b', self.info[0]['flags']) is not None
 
     def _has_sse(self):
-        return re.match(r'.*?\bsse\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bsse\b', self.info[0]['flags']) is not None
 
     def _has_sse2(self):
-        return re.match(r'.*?\bsse2\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bsse2\b', self.info[0]['flags']) is not None
 
     def _has_sse3(self):
-        return re.match(r'.*?\bsse3\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\bpni\b', self.info[0]['flags']) is not None
+
+    def _has_ssse3(self):
+        return re.match(r'.*?\bssse3\b', self.info[0]['flags']) is not None
 
     def _has_3dnow(self):
-        return re.match(r'.*?\b3dnow\b',self.info[0]['flags']) is not None
+        return re.match(r'.*?\b3dnow\b', self.info[0]['flags']) is not None
 
     def _has_3dnowext(self):
-        return re.match(r'.*?\b3dnowext\b',self.info[0]['flags']) is not None
-
-    def _is_64bit(self):
-        if self.is_Alpha():
-            return True
-        if self.info[0].get('clflush size','')=='64':
-            return True
-        if self.info[0].get('uname_m','')=='x86_64':
-            return True
-        if self.info[0].get('arch','')=='IA-64':
-            return True
-        return False
-
-    def _is_32bit(self):
-        return not self.is_64bit()
+        return re.match(r'.*?\b3dnowext\b', self.info[0]['flags']) is not None
 
 class IRIXCPUInfo(CPUInfoBase):
     info = None
@@ -292,7 +305,7 @@ class IRIXCPUInfo(CPUInfoBase):
         if self.info is not None:
             return
         info = key_value_from_command('sysconf', sep=' ',
-                                      successful_status=(0,1))
+                                      successful_status=(0, 1))
         self.__class__.info = info
 
     def _not_impl(self): pass
@@ -303,7 +316,7 @@ class IRIXCPUInfo(CPUInfoBase):
     def _getNCPUs(self):
         return int(self.info.get('NUM_PROCESSORS', 1))
 
-    def __cputype(self,n):
+    def __cputype(self, n):
         return self.info.get('PROCESSORS').split()[0].lower() == 'r%s' % (n)
     def _is_r2000(self): return self.__cputype(2000)
     def _is_r3000(self): return self.__cputype(3000)
@@ -323,8 +336,8 @@ class IRIXCPUInfo(CPUInfoBase):
 
     def get_ip(self):
         try: return self.info.get('MACHINE')
-        except: pass
-    def __machine(self,n):
+        except Exception: pass
+    def __machine(self, n):
         return self.info.get('MACHINE').lower() == 'ip%s' % (n)
     def _is_IP19(self): return self.__machine(19)
     def _is_IP20(self): return self.__machine(20)
@@ -367,7 +380,7 @@ class DarwinCPUInfo(CPUInfoBase):
     def _is_ppc(self):
         return self.info['arch']=='ppc'
 
-    def __machine(self,n):
+    def __machine(self, n):
         return self.info['machine'] == 'ppc%s'%n
     def _is_ppc601(self): return self.__machine(601)
     def _is_ppc602(self): return self.__machine(602)
@@ -412,11 +425,6 @@ class SunOSCPUInfo(CPUInfoBase):
 
     def _not_impl(self): pass
 
-    def _is_32bit(self):
-        return self.info['isainfo_b']=='32'
-    def _is_64bit(self):
-        return self.info['isainfo_b']=='64'
-
     def _is_i386(self):
         return self.info['isainfo_n']=='i386'
     def _is_sparc(self):
@@ -431,35 +439,35 @@ class SunOSCPUInfo(CPUInfoBase):
         return self.info['arch']=='sun4'
 
     def _is_SUNW(self):
-        return re.match(r'SUNW',self.info['uname_i']) is not None
+        return re.match(r'SUNW', self.info['uname_i']) is not None
     def _is_sparcstation5(self):
-        return re.match(r'.*SPARCstation-5',self.info['uname_i']) is not None
+        return re.match(r'.*SPARCstation-5', self.info['uname_i']) is not None
     def _is_ultra1(self):
-        return re.match(r'.*Ultra-1',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-1', self.info['uname_i']) is not None
     def _is_ultra250(self):
-        return re.match(r'.*Ultra-250',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-250', self.info['uname_i']) is not None
     def _is_ultra2(self):
-        return re.match(r'.*Ultra-2',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-2', self.info['uname_i']) is not None
     def _is_ultra30(self):
-        return re.match(r'.*Ultra-30',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-30', self.info['uname_i']) is not None
     def _is_ultra4(self):
-        return re.match(r'.*Ultra-4',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-4', self.info['uname_i']) is not None
     def _is_ultra5_10(self):
-        return re.match(r'.*Ultra-5_10',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-5_10', self.info['uname_i']) is not None
     def _is_ultra5(self):
-        return re.match(r'.*Ultra-5',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-5', self.info['uname_i']) is not None
     def _is_ultra60(self):
-        return re.match(r'.*Ultra-60',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-60', self.info['uname_i']) is not None
     def _is_ultra80(self):
-        return re.match(r'.*Ultra-80',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-80', self.info['uname_i']) is not None
     def _is_ultraenterprice(self):
-        return re.match(r'.*Ultra-Enterprise',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-Enterprise', self.info['uname_i']) is not None
     def _is_ultraenterprice10k(self):
-        return re.match(r'.*Ultra-Enterprise-10000',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra-Enterprise-10000', self.info['uname_i']) is not None
     def _is_sunfire(self):
-        return re.match(r'.*Sun-Fire',self.info['uname_i']) is not None
+        return re.match(r'.*Sun-Fire', self.info['uname_i']) is not None
     def _is_ultra(self):
-        return re.match(r'.*Ultra',self.info['uname_i']) is not None
+        return re.match(r'.*Ultra', self.info['uname_i']) is not None
 
     def _is_cpusparcv7(self):
         return self.info['processor']=='sparcv7'
@@ -482,25 +490,29 @@ class Win32CPUInfo(CPUInfoBase):
         info = []
         try:
             #XXX: Bad style to use so long `try:...except:...`. Fix it!
-            import _winreg
-            prgx = re.compile(r"family\s+(?P<FML>\d+)\s+model\s+(?P<MDL>\d+)"\
-                              "\s+stepping\s+(?P<STP>\d+)",re.IGNORECASE)
-            chnd=_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, self.pkey)
+            if sys.version_info[0] >= 3:
+                import winreg
+            else:
+                import _winreg as winreg
+
+            prgx = re.compile(r"family\s+(?P<FML>\d+)\s+model\s+(?P<MDL>\d+)"
+                              r"\s+stepping\s+(?P<STP>\d+)", re.IGNORECASE)
+            chnd=winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, self.pkey)
             pnum=0
-            while 1:
+            while True:
                 try:
-                    proc=_winreg.EnumKey(chnd,pnum)
-                except _winreg.error:
+                    proc=winreg.EnumKey(chnd, pnum)
+                except winreg.error:
                     break
                 else:
                     pnum+=1
                     info.append({"Processor":proc})
-                    phnd=_winreg.OpenKey(chnd,proc)
+                    phnd=winreg.OpenKey(chnd, proc)
                     pidx=0
                     while True:
                         try:
-                            name,value,vtpe=_winreg.EnumValue(phnd,pidx)
-                        except _winreg.error:
+                            name, value, vtpe=winreg.EnumValue(phnd, pidx)
+                        except winreg.error:
                             break
                         else:
                             pidx=pidx+1
@@ -511,8 +523,8 @@ class Win32CPUInfo(CPUInfoBase):
                                     info[-1]["Family"]=int(srch.group("FML"))
                                     info[-1]["Model"]=int(srch.group("MDL"))
                                     info[-1]["Stepping"]=int(srch.group("STP"))
-        except:
-            print sys.exc_value,'(ignoring)'
+        except Exception:
+            print(sys.exc_info()[1], '(ignoring)')
         self.__class__.info = info
 
     def _not_impl(self): pass
@@ -530,11 +542,11 @@ class Win32CPUInfo(CPUInfoBase):
 
     def _is_AMDK5(self):
         return self.is_AMD() and self.info[0]['Family']==5 \
-               and self.info[0]['Model'] in [0,1,2,3]
+               and self.info[0]['Model'] in [0, 1, 2, 3]
 
     def _is_AMDK6(self):
         return self.is_AMD() and self.info[0]['Family']==5 \
-               and self.info[0]['Model'] in [6,7]
+               and self.info[0]['Model'] in [6, 7]
 
     def _is_AMDK6_2(self):
         return self.is_AMD() and self.info[0]['Family']==5 \
@@ -584,11 +596,11 @@ class Win32CPUInfo(CPUInfoBase):
 
     def _is_PentiumII(self):
         return self.is_Intel() and self.info[0]['Family']==6 \
-               and self.info[0]['Model'] in [3,5,6]
+               and self.info[0]['Model'] in [3, 5, 6]
 
     def _is_PentiumIII(self):
         return self.is_Intel() and self.info[0]['Family']==6 \
-               and self.info[0]['Model'] in [7,8,9,10,11]
+               and self.info[0]['Model'] in [7, 8, 9, 10, 11]
 
     def _is_PentiumIV(self):
         return self.is_Intel() and self.info[0]['Family']==15
@@ -612,20 +624,20 @@ class Win32CPUInfo(CPUInfoBase):
     def _has_mmx(self):
         if self.is_Intel():
             return (self.info[0]['Family']==5 and self.info[0]['Model']==4) \
-                   or (self.info[0]['Family'] in [6,15])
+                   or (self.info[0]['Family'] in [6, 15])
         elif self.is_AMD():
-            return self.info[0]['Family'] in [5,6,15]
+            return self.info[0]['Family'] in [5, 6, 15]
         else:
             return False
 
     def _has_sse(self):
         if self.is_Intel():
             return (self.info[0]['Family']==6 and \
-                    self.info[0]['Model'] in [7,8,9,10,11]) \
+                    self.info[0]['Model'] in [7, 8, 9, 10, 11]) \
                     or self.info[0]['Family']==15
         elif self.is_AMD():
             return (self.info[0]['Family']==6 and \
-                    self.info[0]['Model'] in [6,7,8,10]) \
+                    self.info[0]['Model'] in [6, 7, 8, 10]) \
                     or self.info[0]['Family']==15
         else:
             return False
@@ -640,10 +652,10 @@ class Win32CPUInfo(CPUInfoBase):
             return False
 
     def _has_3dnow(self):
-        return self.is_AMD() and self.info[0]['Family'] in [5,6,15]
+        return self.is_AMD() and self.info[0]['Family'] in [5, 6, 15]
 
     def _has_3dnowext(self):
-        return self.is_AMD() and self.info[0]['Family'] in [6,15]
+        return self.is_AMD() and self.info[0]['Family'] in [6, 15]
 
 if sys.platform.startswith('linux'): # variations: linux2,linux-i386 (any others?)
     cpuinfo = LinuxCPUInfo
@@ -663,19 +675,19 @@ else:
 
 cpu = cpuinfo()
 
-if __name__ == "__main__":
-
-    cpu.is_blaa()
-    cpu.is_Intel()
-    cpu.is_Alpha()
-
-    print 'CPU information:',
-    for name in dir(cpuinfo):
-        if name[0]=='_' and name[1]!='_':
-            r = getattr(cpu,name[1:])()
-            if r:
-                if r!=1:
-                    print '%s=%s' %(name[1:],r),
-                else:
-                    print name[1:],
-    print
+#if __name__ == "__main__":
+#
+#    cpu.is_blaa()
+#    cpu.is_Intel()
+#    cpu.is_Alpha()
+#
+#    print('CPU information:'),
+#    for name in dir(cpuinfo):
+#        if name[0]=='_' and name[1]!='_':
+#            r = getattr(cpu,name[1:])()
+#            if r:
+#                if r!=1:
+#                    print('%s=%s' %(name[1:],r))
+#                else:
+#                    print(name[1:]),
+#    print()
