@@ -7,9 +7,9 @@ Support code for building Python extensions on Windows.
     # 3. Force windows to use g77
 
 """
+from __future__ import division, absolute_import, print_function
 
 import os
-import subprocess
 import sys
 import subprocess
 import re
@@ -18,7 +18,7 @@ import re
 import numpy.distutils.ccompiler
 
 if sys.version_info[0] < 3:
-    import log
+    from . import log
 else:
     from numpy.distutils import log
 # NT stuff
@@ -30,11 +30,23 @@ else:
 import distutils.cygwinccompiler
 from distutils.version import StrictVersion
 from numpy.distutils.ccompiler import gen_preprocess_options, gen_lib_options
-from distutils.errors import DistutilsExecError, CompileError, UnknownFileError
-
 from distutils.unixccompiler import UnixCCompiler
 from distutils.msvccompiler import get_build_version as get_build_msvc_version
-from numpy.distutils.misc_util import msvc_runtime_library, get_build_architecture
+from distutils.errors import (DistutilsExecError, CompileError,
+                              UnknownFileError)
+from numpy.distutils.misc_util import (msvc_runtime_library,
+                                       msvc_runtime_version,
+                                       msvc_runtime_major,
+                                       get_build_architecture)
+
+def get_msvcr_replacement():
+    """Replacement for outdated version of get_msvcr from cygwinccompiler"""
+    msvcr = msvc_runtime_library()
+    return [] if msvcr is None else [msvcr]
+
+# monkey-patch cygwinccompiler with our updated version from misc_util
+# to avoid getting an exception raised on Python 3.5
+distutils.cygwinccompiler.get_msvcr = get_msvcr_replacement
 
 # Useful to generate table of symbols from a dll
 _START = re.compile(r'\[Ordinal/Name Pointer\] Table')
@@ -53,18 +65,17 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
                   dry_run=0,
                   force=0):
 
-        distutils.cygwinccompiler.CygwinCCompiler.__init__ (self,
-                                                       verbose,dry_run, force)
+        distutils.cygwinccompiler.CygwinCCompiler.__init__ (self, verbose,
+                                                            dry_run, force)
 
         # we need to support 3.2 which doesn't match the standard
         # get_versions methods regex
         if self.gcc_version is None:
-            import re
             p = subprocess.Popen(['gcc', '-dumpversion'], shell=True,
                                  stdout=subprocess.PIPE)
             out_string = p.stdout.read()
             p.stdout.close()
-            result = re.search('(\d+\.\d+)',out_string)
+            result = re.search(r'(\d+\.\d+)', out_string)
             if result:
                 self.gcc_version = StrictVersion(result.group(1))
 
@@ -80,7 +91,8 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
             #   g++.exe: g++: No such file or directory
             # error (mingw 1.0 in Enthon24 tree, gcc-3.4.5).
             # If the --driver-name part is required for some environment
-            # then make the inclusion of this part specific to that environment.
+            # then make the inclusion of this part specific to that
+            # environment.
             self.linker = 'dllwrap' #  --driver-name g++'
         elif self.linker_dll == 'gcc':
             self.linker = 'g++'
@@ -90,24 +102,28 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
 
         build_import_library()
 
-        # **changes: eric jones 4/11/01
-        # 2. increased optimization and turned off all warnings
-        # 3. also added --driver-name g++
-        #self.set_executables(compiler='gcc -mno-cygwin -O2 -w',
-        #                     compiler_so='gcc -mno-cygwin -mdll -O2 -w',
-        #                     linker_exe='gcc -mno-cygwin',
-        #                     linker_so='%s --driver-name g++ -mno-cygwin -mdll -static %s'
-        #                                % (self.linker, entry_point))
+        # Check for custom msvc runtime library on Windows. Build if it doesn't exist.
+        msvcr_success = build_msvcr_library()
+        msvcr_dbg_success = build_msvcr_library(debug=True)
+        if msvcr_success or msvcr_dbg_success:
+            # add preprocessor statement for using customized msvcr lib
+            self.define_macro('NPY_MINGW_USE_CUSTOM_MSVCR')
 
-        # MS_WIN64 should be defined when building for amd64 on windows, but
-        # python headers define it only for MS compilers, which has all kind of
-        # bad consequences, like using Py_ModuleInit4 instead of
+        # Define the MSVC version as hint for MinGW
+        msvcr_version = msvc_runtime_version()
+        if msvcr_version:
+            self.define_macro('__MSVCRT_VERSION__', '0x%04i' % msvcr_version)
+
+        # MS_WIN64 should be defined when building for amd64 on windows,
+        # but python headers define it only for MS compilers, which has all
+        # kind of bad consequences, like using Py_ModuleInit4 instead of
         # Py_ModuleInit4_64, etc... So we add it here
         if get_build_architecture() == 'AMD64':
-            if self.gcc_version < "4.":
+            if self.gcc_version < "4.0":
                 self.set_executables(
                     compiler='gcc -g -DDEBUG -DMS_WIN64 -mno-cygwin -O0 -Wall',
-                    compiler_so='gcc -g -DDEBUG -DMS_WIN64 -mno-cygwin -O0 -Wall -Wstrict-prototypes',
+                    compiler_so='gcc -g -DDEBUG -DMS_WIN64 -mno-cygwin -O0'
+                                ' -Wall -Wstrict-prototypes',
                     linker_exe='gcc -g -mno-cygwin',
                     linker_so='gcc -g -mno-cygwin -shared')
             else:
@@ -119,16 +135,20 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
                     linker_so='gcc -g -shared')
         else:
             if self.gcc_version <= "3.0.0":
-                self.set_executables(compiler='gcc -mno-cygwin -O2 -w',
-                                     compiler_so='gcc -mno-cygwin -mdll -O2 -w -Wstrict-prototypes',
-                                     linker_exe='g++ -mno-cygwin',
-                                     linker_so='%s -mno-cygwin -mdll -static %s'
-                                     % (self.linker, entry_point))
-            elif self.gcc_version < "4.":
-                self.set_executables(compiler='gcc -mno-cygwin -O2 -Wall',
-                                     compiler_so='gcc -mno-cygwin -O2 -Wall -Wstrict-prototypes',
-                                     linker_exe='g++ -mno-cygwin',
-                                     linker_so='g++ -mno-cygwin -shared')
+                self.set_executables(
+                    compiler='gcc -mno-cygwin -O2 -w',
+                    compiler_so='gcc -mno-cygwin -mdll -O2 -w'
+                                ' -Wstrict-prototypes',
+                    linker_exe='g++ -mno-cygwin',
+                    linker_so='%s -mno-cygwin -mdll -static %s' %
+                              (self.linker, entry_point))
+            elif self.gcc_version < "4.0":
+                self.set_executables(
+                    compiler='gcc -mno-cygwin -O2 -Wall',
+                    compiler_so='gcc -mno-cygwin -O2 -Wall'
+                                ' -Wstrict-prototypes',
+                    linker_exe='g++ -mno-cygwin',
+                    linker_so='g++ -mno-cygwin -shared')
             else:
                 # gcc-4 series releases do not support -mno-cygwin option
                 self.set_executables(compiler='gcc -O2 -Wall',
@@ -139,9 +159,9 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
         # we can't pass it through set_executables because pre 2.2 would fail
         self.compiler_cxx = ['g++']
 
-        # Maybe we should also append -mthreads, but then the finished
-        # dlls need another dll (mingwm10.dll see Mingw32 docs)
-        # (-mthreads: Support thread-safe exception handling on `Mingw32')
+        # Maybe we should also append -mthreads, but then the finished dlls
+        # need another dll (mingwm10.dll see Mingw32 docs) (-mthreads: Support
+        # thread-safe exception handling on `Mingw32')
 
         # no additional libraries needed
         #self.dll_libraries=[]
@@ -163,7 +183,7 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
              extra_postargs=None,
              build_temp=None,
              target_lang=None):
-        # Include the appropiate MSVC runtime library if Python was built
+        # Include the appropriate MSVC runtime library if Python was built
         # with MSVC >= 7.0 (MinGW standard is msvcrt)
         runtime_library = msvc_runtime_library()
         if runtime_library:
@@ -188,10 +208,7 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
             func = distutils.cygwinccompiler.CygwinCCompiler.link
         else:
             func = UnixCCompiler.link
-        if sys.version_info[0] >= 3:
-            func(*args[:func.__code__.co_argcount])
-        else:
-            func(*args[:func.im_func.func_code.co_argcount])
+        func(*args[:func.__code__.co_argcount])
         return
 
     def object_filenames (self,
@@ -207,11 +224,11 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
             # added these lines to strip off windows drive letters
             # without it, .o files are placed next to .c files
             # instead of the build directory
-            drv,base = os.path.splitdrive(base)
+            drv, base = os.path.splitdrive(base)
             if drv:
                 base = base[1:]
 
-            if ext not in (self.src_extensions + ['.rc','.res']):
+            if ext not in (self.src_extensions + ['.rc', '.res']):
                 raise UnknownFileError(
                       "unknown file type '%s' (from '%s')" % \
                       (ext, src_name))
@@ -230,25 +247,40 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
 
 
 def find_python_dll():
-    maj, min, micro = [int(i) for i in sys.version_info[:3]]
-    dllname = 'python%d%d.dll' % (maj, min)
-    print ("Looking for %s" % dllname)
-
     # We can't do much here:
-    # - find it in python main dir
+    # - find it in the virtualenv (sys.prefix)
+    # - find it in python main dir (sys.base_prefix, if in a virtualenv)
+    # - sys.real_prefix is main dir for virtualenvs in Python 2.7
     # - in system32,
     # - ortherwise (Sxs), I don't know how to get it.
-    lib_dirs = []
-    lib_dirs.append(os.path.join(sys.prefix, 'lib'))
-    try:
-        lib_dirs.append(os.path.join(os.environ['SYSTEMROOT'], 'system32'))
-    except KeyError:
-        pass
+    stems = [sys.prefix]
+    if hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix:
+        stems.append(sys.base_prefix)
+    elif hasattr(sys, 'real_prefix') and sys.real_prefix != sys.prefix:
+        stems.append(sys.real_prefix)
 
-    for d in lib_dirs:
-        dll = os.path.join(d, dllname)
-        if os.path.exists(dll):
-            return dll
+    sub_dirs = ['', 'lib', 'bin']
+    # generate possible combinations of directory trees and sub-directories
+    lib_dirs = []
+    for stem in stems:
+        for folder in sub_dirs:
+            lib_dirs.append(os.path.join(stem, folder))
+
+    # add system directory as well
+    if 'SYSTEMROOT' in os.environ:
+        lib_dirs.append(os.path.join(os.environ['SYSTEMROOT'], 'System32'))
+
+    # search in the file system for possible candidates
+    major_version, minor_version = tuple(sys.version_info[:2])
+    patterns = ['python%d%d.dll']
+
+    for pat in patterns:
+        dllname = pat % (major_version, minor_version)
+        print("Looking for %s" % dllname)
+        for folder in lib_dirs:
+            dll = os.path.join(folder, dllname)
+            if os.path.exists(dll):
+                return dll
 
     raise ValueError("%s not found in %s" % (dllname, lib_dirs))
 
@@ -263,15 +295,14 @@ def generate_def(dll, dfile):
     The .def file will be overwritten"""
     dump = dump_table(dll)
     for i in range(len(dump)):
-        if _START.match(dump[i]):
+        if _START.match(dump[i].decode()):
             break
-
-    if i == len(dump):
+    else:
         raise ValueError("Symbol table not found")
 
     syms = []
     for j in range(i+1, len(dump)):
-        m = _TABLE.match(dump[j])
+        m = _TABLE.match(dump[j].decode())
         if m:
             syms.append((int(m.group(1).strip()), m.group(2)))
         else:
@@ -290,6 +321,91 @@ def generate_def(dll, dfile):
         d.write('%s\n' % s[1])
     d.close()
 
+def find_dll(dll_name):
+
+    arch = {'AMD64' : 'amd64',
+            'Intel' : 'x86'}[get_build_architecture()]
+
+    def _find_dll_in_winsxs(dll_name):
+        # Walk through the WinSxS directory to find the dll.
+        winsxs_path = os.path.join(os.environ.get('WINDIR', r'C:\WINDOWS'),
+                                   'winsxs')
+        if not os.path.exists(winsxs_path):
+            return None
+        for root, dirs, files in os.walk(winsxs_path):
+            if dll_name in files and arch in root:
+                return os.path.join(root, dll_name)
+        return None
+
+    def _find_dll_in_path(dll_name):
+        # First, look in the Python directory, then scan PATH for
+        # the given dll name.
+        for path in [sys.prefix] + os.environ['PATH'].split(';'):
+            filepath = os.path.join(path, dll_name)
+            if os.path.exists(filepath):
+                return os.path.abspath(filepath)
+
+    return _find_dll_in_winsxs(dll_name) or _find_dll_in_path(dll_name)
+
+def build_msvcr_library(debug=False):
+    if os.name != 'nt':
+        return False
+
+    # If the version number is None, then we couldn't find the MSVC runtime at
+    # all, because we are running on a Python distribution which is customed
+    # compiled; trust that the compiler is the same as the one available to us
+    # now, and that it is capable of linking with the correct runtime without
+    # any extra options.
+    msvcr_ver = msvc_runtime_major()
+    if msvcr_ver is None:
+        log.debug('Skip building import library: '
+                  'Runtime is not compiled with MSVC')
+        return False
+
+    # Skip using a custom library for versions < MSVC 8.0
+    if msvcr_ver < 80:
+        log.debug('Skip building msvcr library:'
+                  ' custom functionality not present')
+        return False
+
+    msvcr_name = msvc_runtime_library()
+    if debug:
+        msvcr_name += 'd'
+
+    # Skip if custom library already exists
+    out_name = "lib%s.a" % msvcr_name
+    out_file = os.path.join(sys.prefix, 'libs', out_name)
+    if os.path.isfile(out_file):
+        log.debug('Skip building msvcr library: "%s" exists' %
+                  (out_file,))
+        return True
+
+    # Find the msvcr dll
+    msvcr_dll_name = msvcr_name + '.dll'
+    dll_file = find_dll(msvcr_dll_name)
+    if not dll_file:
+        log.warn('Cannot build msvcr library: "%s" not found' %
+                 msvcr_dll_name)
+        return False
+
+    def_name = "lib%s.def" % msvcr_name
+    def_file = os.path.join(sys.prefix, 'libs', def_name)
+
+    log.info('Building msvcr library: "%s" (from %s)' \
+             % (out_file, dll_file))
+
+    # Generate a symbol definition file from the msvcr dll
+    generate_def(dll_file, def_file)
+
+    # Create a custom mingw library for the given symbol definitions
+    cmd = ['dlltool', '-d', def_file, '-l', out_file]
+    retcode = subprocess.call(cmd)
+
+    # Clean up symbol definitions
+    os.remove(def_file)
+
+    return (not retcode)
+
 def build_import_library():
     if os.name != 'nt':
         return
@@ -302,60 +418,106 @@ def build_import_library():
     else:
         raise ValueError("Unhandled arch %s" % arch)
 
-def _build_import_library_amd64():
-    dll_file = find_python_dll()
+def _check_for_import_lib():
+    """Check if an import library for the Python runtime already exists."""
+    major_version, minor_version = tuple(sys.version_info[:2])
 
-    out_name = "libpython%d%d.a" % tuple(sys.version_info[:2])
-    out_file = os.path.join(sys.prefix, 'libs', out_name)
-    if os.path.isfile(out_file):
-        log.debug('Skip building import library: "%s" exists' % (out_file))
+    # patterns for the file name of the library itself
+    patterns = ['libpython%d%d.a',
+                'libpython%d%d.dll.a',
+                'libpython%d.%d.dll.a']
+
+    # directory trees that may contain the library
+    stems = [sys.prefix]
+    if hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix:
+        stems.append(sys.base_prefix)
+    elif hasattr(sys, 'real_prefix') and sys.real_prefix != sys.prefix:
+        stems.append(sys.real_prefix)
+
+    # possible subdirectories within those trees where it is placed
+    sub_dirs = ['libs', 'lib']
+
+    # generate a list of candidate locations
+    candidates = []
+    for pat in patterns:
+        filename = pat % (major_version, minor_version)
+        for stem_dir in stems:
+            for folder in sub_dirs:
+                candidates.append(os.path.join(stem_dir, folder, filename))
+
+    # test the filesystem to see if we can find any of these
+    for fullname in candidates:
+        if os.path.isfile(fullname):
+            # already exists, in location given
+            return (True, fullname)
+
+    # needs to be built, preferred location given first
+    return (False, candidates[0])
+
+def _build_import_library_amd64():
+    out_exists, out_file = _check_for_import_lib()
+    if out_exists:
+        log.debug('Skip building import library: "%s" exists', out_file)
         return
 
+    # get the runtime dll for which we are building import library
+    dll_file = find_python_dll()
+    log.info('Building import library (arch=AMD64): "%s" (from %s)' %
+             (out_file, dll_file))
+
+    # generate symbol list from this library
     def_name = "python%d%d.def" % tuple(sys.version_info[:2])
-    def_file = os.path.join(sys.prefix,'libs',def_name)
-
-    log.info('Building import library (arch=AMD64): "%s" (from %s)' \
-             % (out_file, dll_file))
-
+    def_file = os.path.join(sys.prefix, 'libs', def_name)
     generate_def(dll_file, def_file)
 
+    # generate import library from this symbol list
     cmd = ['dlltool', '-d', def_file, '-l', out_file]
     subprocess.Popen(cmd)
 
 def _build_import_library_x86():
     """ Build the import libraries for Mingw32-gcc on Windows
     """
+    out_exists, out_file = _check_for_import_lib()
+    if out_exists:
+        log.debug('Skip building import library: "%s" exists', out_file)
+        return
+
     lib_name = "python%d%d.lib" % tuple(sys.version_info[:2])
-    lib_file = os.path.join(sys.prefix,'libs',lib_name)
-    out_name = "libpython%d%d.a" % tuple(sys.version_info[:2])
-    out_file = os.path.join(sys.prefix,'libs',out_name)
+    lib_file = os.path.join(sys.prefix, 'libs', lib_name)
     if not os.path.isfile(lib_file):
-        log.warn('Cannot build import library: "%s" not found' % (lib_file))
-        return
-    if os.path.isfile(out_file):
-        log.debug('Skip building import library: "%s" exists' % (out_file))
-        return
-    log.info('Building import library (ARCH=x86): "%s"' % (out_file))
+        # didn't find library file in virtualenv, try base distribution, too,
+        # and use that instead if found there. for Python 2.7 venvs, the base
+        # directory is in attribute real_prefix instead of base_prefix.
+        if hasattr(sys, 'base_prefix'):
+            base_lib = os.path.join(sys.base_prefix, 'libs', lib_name)
+        elif hasattr(sys, 'real_prefix'):
+            base_lib = os.path.join(sys.real_prefix, 'libs', lib_name)
+        else:
+            base_lib = ''  # os.path.isfile('') == False
+
+        if os.path.isfile(base_lib):
+            lib_file = base_lib
+        else:
+            log.warn('Cannot build import library: "%s" not found', lib_file)
+            return
+    log.info('Building import library (ARCH=x86): "%s"', out_file)
 
     from numpy.distutils import lib2def
 
     def_name = "python%d%d.def" % tuple(sys.version_info[:2])
-    def_file = os.path.join(sys.prefix,'libs',def_name)
+    def_file = os.path.join(sys.prefix, 'libs', def_name)
     nm_cmd = '%s %s' % (lib2def.DEFAULT_NM, lib_file)
     nm_output = lib2def.getnm(nm_cmd)
     dlist, flist = lib2def.parse_nm(nm_output)
     lib2def.output_def(dlist, flist, lib2def.DEF_HEADER, open(def_file, 'w'))
 
-    dll_name = "python%d%d.dll" % tuple(sys.version_info[:2])
-    args = (dll_name,def_file,out_file)
-    cmd = 'dlltool --dllname %s --def %s --output-lib %s' % args
+    dll_name = find_python_dll ()
+    args = (dll_name, def_file, out_file)
+    cmd = 'dlltool --dllname "%s" --def "%s" --output-lib "%s"' % args
     status = os.system(cmd)
     # for now, fail silently
     if status:
         log.warn('Failed to build import library for gcc. Linking will fail.')
-    #if not success:
-    #    msg = "Couldn't find import library, and failed to build it."
-    #    raise DistutilsPlatformError, msg
     return
 
 #=====================================
@@ -379,17 +541,21 @@ _MSVCRVER_TO_FULLVER = {}
 if sys.platform == 'win32':
     try:
         import msvcrt
-        if hasattr(msvcrt, "CRT_ASSEMBLY_VERSION"):
-            _MSVCRVER_TO_FULLVER['90'] = msvcrt.CRT_ASSEMBLY_VERSION
-        else:
-            _MSVCRVER_TO_FULLVER['90'] = "9.0.21022.8"
         # I took one version in my SxS directory: no idea if it is the good
         # one, and we can't retrieve it from python
         _MSVCRVER_TO_FULLVER['80'] = "8.0.50727.42"
+        _MSVCRVER_TO_FULLVER['90'] = "9.0.21022.8"
+        # Value from msvcrt.CRT_ASSEMBLY_VERSION under Python 3.3.0
+        # on Windows XP:
+        _MSVCRVER_TO_FULLVER['100'] = "10.0.30319.460"
+        if hasattr(msvcrt, "CRT_ASSEMBLY_VERSION"):
+            major, minor, rest = msvcrt.CRT_ASSEMBLY_VERSION.split(".", 2)
+            _MSVCRVER_TO_FULLVER[major + minor] = msvcrt.CRT_ASSEMBLY_VERSION
+            del major, minor, rest
     except ImportError:
-        # If we are here, means python was not built with MSVC. Not sure what to do
-        # in that case: manifest building will fail, but it should not be used in
-        # that case anyway
+        # If we are here, means python was not built with MSVC. Not sure what
+        # to do in that case: manifest building will fail, but it should not be
+        # used in that case anyway
         log.warn('Cannot import msvcrt: using manifest will not be possible')
 
 def msvc_manifest_xml(maj, min):
@@ -398,11 +564,11 @@ def msvc_manifest_xml(maj, min):
     try:
         fullver = _MSVCRVER_TO_FULLVER[str(maj * 10 + min)]
     except KeyError:
-        raise ValueError("Version %d,%d of MSVCRT not supported yet" \
-                         % (maj, min))
+        raise ValueError("Version %d,%d of MSVCRT not supported yet" %
+                         (maj, min))
     # Don't be fooled, it looks like an XML, but it is not. In particular, it
     # should not have any space before starting, and its size should be
-    # divisible by 4, most likely for alignement constraints when the xml is
+    # divisible by 4, most likely for alignment constraints when the xml is
     # embedded in the binary...
     # This template was copied directly from the python 2.6 binary (using
     # strings.exe from mingw on python.exe).
@@ -430,10 +596,13 @@ def manifest_rc(name, type='dll'):
     'exe').
 
     Parameters
-    ---------- name: str
+    ----------
+    name : str
             name of the manifest file to embed
-        type: str ('dll', 'exe')
-            type of the binary which will embed the manifest"""
+    type : str {'dll', 'exe'}
+            type of the binary which will embed the manifest
+
+    """
     if type == 'dll':
         rctype = 2
     elif type == 'exe':
@@ -449,9 +618,8 @@ def check_embedded_msvcr_match_linked(msver):
     """msver is the ms runtime version used for the MANIFEST."""
     # check msvcr major version are the same for linking and
     # embedding
-    msvcv = msvc_runtime_library()
-    if msvcv:
-        maj = int(msvcv[5:6])
+    maj = msvc_runtime_major()
+    if maj:
         if not maj == int(msver):
             raise ValueError(
                   "Discrepancy between linked msvcr " \
@@ -469,7 +637,7 @@ def manifest_name(config):
     return root + exext + ".manifest"
 
 def rc_name(config):
-    # Get configest name (including suffix)
+    # Get configtest name (including suffix)
     root = configtest_name(config)
     return root + ".rc"
 
@@ -486,9 +654,3 @@ def generate_manifest(config):
             config.temp_files.append(manifest_name(config))
             man.write(manxml)
             man.close()
-            # # Write the rc file
-            # manrc = manifest_rc(manifest_name(self), "exe")
-            # rc = open(rc_name(self), "w")
-            # self.temp_files.append(manrc)
-            # rc.write(manrc)
-            # rc.close()
