@@ -9,7 +9,7 @@ extern "C" {
 #ifdef FORTRANOBJECT_C
 #define NO_IMPORT_ARRAY
 #endif
-#define PY_ARRAY_UNIQUE_SYMBOL PyArray_API
+#define PY_ARRAY_UNIQUE_SYMBOL _npy_f2py_ARRAY_API
 #include "numpy/arrayobject.h"
 
 /*
@@ -20,6 +20,7 @@ extern "C" {
 #define PyString_GET_SIZE PyBytes_GET_SIZE
 #define PyString_AS_STRING PyBytes_AS_STRING
 #define PyString_FromString PyBytes_FromString
+#define PyUString_FromStringAndSize PyUnicode_FromStringAndSize
 #define PyString_ConcatAndDel PyBytes_ConcatAndDel
 #define PyString_AsString PyBytes_AsString
 
@@ -29,29 +30,12 @@ extern "C" {
 #define PyInt_AsLong PyLong_AsLong
 
 #define PyNumber_Int PyNumber_Long
-#endif
 
-#if (PY_VERSION_HEX < 0x02060000)
-#define Py_TYPE(o)    (((PyObject*)(o))->ob_type)
-#define Py_REFCNT(o)  (((PyObject*)(o))->ob_refcnt)
-#define Py_SIZE(o)    (((PyVarObject*)(o))->ob_size)
-#endif
-
-  /*
-#ifdef F2PY_REPORT_ATEXIT_DISABLE
-#undef F2PY_REPORT_ATEXIT
 #else
 
-#ifndef __FreeBSD__
-#ifndef __WIN32__
-#ifndef __APPLE__
-#define F2PY_REPORT_ATEXIT
-#endif
-#endif
+#define PyUString_FromStringAndSize PyString_FromStringAndSize
 #endif
 
-#endif
-  */
 
 #ifdef F2PY_REPORT_ATEXIT
 #include <sys/timeb.h>
@@ -94,16 +78,16 @@ typedef void *(*f2pycfunc)(void);
 typedef struct {
   char *name;                /* attribute (array||routine) name */
   int rank;                  /* array rank, 0 for scalar, max is F2PY_MAX_DIMS,
-				|| rank=-1 for Fortran routine */
+                                || rank=-1 for Fortran routine */
   struct {npy_intp d[F2PY_MAX_DIMS];} dims; /* dimensions of the array, || not used */
   int type;                  /* PyArray_<type> || not used */
   char *data;                /* pointer to array || Fortran routine */
-  f2py_init_func func;            /* initialization function for
-				allocatable arrays:
-				func(&rank,dims,set_ptr_func,name,len(name))
-				|| C/API wrapper for Fortran routine */
+  f2py_init_func func;       /* initialization function for
+                                allocatable arrays:
+                                func(&rank,dims,set_ptr_func,name,len(name))
+                                || C/API wrapper for Fortran routine */
   char *doc;                 /* documentation string; only recommended
-				for routines. */
+                                for routines. */
 } FortranDataDef;
 
 typedef struct {
@@ -135,7 +119,7 @@ int F2PyCapsule_Check(PyObject *ptr);
 
 #endif
 
-#define ISCONTIGUOUS(m) ((m)->flags & NPY_CONTIGUOUS)
+#define ISCONTIGUOUS(m) (PyArray_FLAGS(m) & NPY_ARRAY_C_CONTIGUOUS)
 #define F2PY_INTENT_IN 1
 #define F2PY_INTENT_INOUT 2
 #define F2PY_INTENT_OUT 4
@@ -155,16 +139,16 @@ int F2PyCapsule_Check(PyObject *ptr);
 #define F2PY_ALIGN16(intent) (intent & F2PY_INTENT_ALIGNED16)
 
 #define F2PY_GET_ALIGNMENT(intent) \
-	(F2PY_ALIGN4(intent) ? 4 : \
-	 (F2PY_ALIGN8(intent) ? 8 : \
-	  (F2PY_ALIGN16(intent) ? 16 : 1) ))
+        (F2PY_ALIGN4(intent) ? 4 : \
+         (F2PY_ALIGN8(intent) ? 8 : \
+          (F2PY_ALIGN16(intent) ? 16 : 1) ))
 #define F2PY_CHECK_ALIGNMENT(arr, intent) ARRAY_ISALIGNED(arr, F2PY_GET_ALIGNMENT(intent))
 
   extern PyArrayObject* array_from_pyobj(const int type_num,
-					 npy_intp *dims,
-					 const int rank,
-					 const int intent,
-					 PyObject *obj);
+                                         npy_intp *dims,
+                                         const int rank,
+                                         const int intent,
+                                         PyObject *obj);
   extern int copy_ND_array(const PyArrayObject *in, PyArrayObject *out);
 
 #ifdef DEBUG_COPY_ND_ARRAY
