@@ -1,9 +1,9 @@
+from __future__ import division, print_function
 
-from os.path import join
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
-    config = Configuration('fft',parent_package,top_path)
+    config = Configuration('fft', parent_package, top_path)
 
     config.add_data_dir('tests')
 
@@ -12,9 +12,8 @@ def configuration(parent_package='',top_path=None):
                          sources=['fftpack_litemodule.c', 'fftpack.c']
                          )
 
-
     return config
 
 if __name__ == '__main__':
     from numpy.distutils.core import setup
-    setup(**configuration(top_path='').todict())
+    setup(configuration=configuration)
