@@ -1,12 +1,13 @@
+from __future__ import division, absolute_import, print_function
+
 from operator import mul
+from functools import reduce
 
 import numpy as np
 from numpy.random import randint
 from numpy.lib import Arrayterator
+from numpy.testing import assert_
 
-import sys
-if sys.version_info[0] >= 3:
-    from functools import reduce
 
 def test():
     np.random.seed(np.arange(10))
@@ -23,10 +24,10 @@ def test():
 
     # Check that each block has at most ``buf_size`` elements
     for block in b:
-        assert len(block.flat) <= (buf_size or els)
+        assert_(len(block.flat) <= (buf_size or els))
 
     # Check that all elements are iterated correctly
-    assert list(b.flat) == list(a.flat)
+    assert_(list(b.flat) == list(a.flat))
 
     # Slice arrayterator
     start = [randint(dim) for dim in shape]
@@ -38,14 +39,10 @@ def test():
 
     # Check that each block has at most ``buf_size`` elements
     for block in c:
-        assert len(block.flat) <= (buf_size or els)
+        assert_(len(block.flat) <= (buf_size or els))
 
     # Check that the arrayterator is sliced correctly
-    assert np.all(c.__array__() == d)
+    assert_(np.all(c.__array__() == d))
 
     # Check that all elements are iterated correctly
-    assert list(c.flat) == list(d.flat)
-
-if __name__ == '__main__':
-    from numpy.testing import run_module_suite
-    run_module_suite()
+    assert_(list(c.flat) == list(d.flat))
